@@ -35,16 +35,19 @@ from tlz import (
     valmap,
 )
 
-from .. import config
-from ..base import DaskMethodsMixin, dont_optimize, replace_name_in_key, tokenize
-from ..context import globalmethod
-from ..core import flatten, get_dependencies, istask, quote, reverse_dict
-from ..delayed import Delayed, unpack_collections
-from ..highlevelgraph import HighLevelGraph
-from ..multiprocessing import get as mpget
-from ..optimization import cull, fuse, inline
-from ..sizeof import sizeof
-from ..utils import (
+from dask import config
+from dask.bag import chunk
+from dask.bag.avro import to_avro
+from dask.base import DaskMethodsMixin, dont_optimize, replace_name_in_key, tokenize
+from dask.blockwise import blockwise
+from dask.context import globalmethod
+from dask.core import flatten, get_dependencies, istask, quote, reverse_dict
+from dask.delayed import Delayed, unpack_collections
+from dask.highlevelgraph import HighLevelGraph
+from dask.multiprocessing import get as mpget
+from dask.optimization import cull, fuse, inline
+from dask.sizeof import sizeof
+from dask.utils import (
     apply,
     digit,
     ensure_bytes,
@@ -58,8 +61,6 @@ from ..utils import (
     system_encoding,
     takes_multiple_arguments,
 )
-from . import chunk
-from .avro import to_avro
 
 no_default = "__no__default__"
 no_result = type(
@@ -92,7 +93,7 @@ def lazify_task(task, start=True):
         task = task[1]
         return lazify_task(*tail, start=False)
     else:
-        return (head,) + tuple([lazify_task(arg, False) for arg in tail])
+        return (head,) + tuple(lazify_task(arg, False) for arg in tail)
 
 
 def lazify(dsk):
@@ -173,7 +174,7 @@ def to_textfiles(
     compute=True,
     storage_options=None,
     last_endline=False,
-    **kwargs
+    **kwargs,
 ):
     """Write dask Bag to disk, one filename per partition, one line per element.
 
@@ -239,7 +240,7 @@ def to_textfiles(
         encoding=encoding,
         name_function=name_function,
         num=b.npartitions,
-        **(storage_options or {})
+        **(storage_options or {}),
     )
 
     name = "to-textfiles-" + uuid.uuid4().hex
@@ -341,11 +342,20 @@ def robust_wraps(wrapper):
 
 
 class Item(DaskMethodsMixin):
-    def __init__(self, dsk, key):
+    def __init__(self, dsk, key, layer=None):
         self.dask = dsk
         self.key = key
         self.name = key
 
+        # NOTE: Layer only used by `Item.from_delayed`, to handle Delayed objects created by other collections.
+        # e.g.: Item.from_delayed(da.ones(1).to_delayed()[0])
+        # See Delayed.__init__
+        self._layer = layer or key
+        if isinstance(dsk, HighLevelGraph) and self._layer not in dsk.layers:
+            raise ValueError(
+                f"Layer {self._layer} not in the HighLevelGraph's layers: {list(dsk.layers)}"
+            )
+
     def __dask_graph__(self):
         return self.dask
 
@@ -353,13 +363,7 @@ class Item(DaskMethodsMixin):
         return [self.key]
 
     def __dask_layers__(self):
-        # Deal with instances created with Item.from_delayed()
-        # e.g.: Item.from_delayed(da.ones(1).to_delayed()[0])
-        # See Delayed.__dask_layers__
-        if isinstance(self.dask, HighLevelGraph) and len(self.dask.layers) == 1:
-            return tuple(self.dask.layers)
-        else:
-            return (self.key,)
+        return (self._layer,)
 
     def __dask_tokenize__(self):
         return self.key
@@ -388,7 +392,7 @@ class Item(DaskMethodsMixin):
         if not isinstance(value, Delayed) and hasattr(value, "key"):
             value = delayed(value)
         assert isinstance(value, Delayed)
-        return Item(value.dask, value.key)
+        return Item(value.dask, value.key, layer=value.__dask_layers__()[0])
 
     @property
     def _args(self):
@@ -401,7 +405,7 @@ class Item(DaskMethodsMixin):
         self.dask, self.key = state
 
     def apply(self, func):
-        name = "{0}-{1}".format(funcname(func), tokenize(self, func, "apply"))
+        name = "{}-{}".format(funcname(func), tokenize(self, func, "apply"))
         dsk = {name: (func, self.key)}
         graph = HighLevelGraph.from_collections(name, dsk, dependencies=[self])
         return Item(graph, name)
@@ -422,7 +426,7 @@ class Item(DaskMethodsMixin):
         dsk = self.__dask_graph__()
         if optimize_graph:
             dsk = self.__dask_optimize__(dsk, self.__dask_keys__())
-        return Delayed(self.key, dsk)
+        return Delayed(self.key, dsk, layer=self._layer)
 
 
 class Bag(DaskMethodsMixin):
@@ -602,9 +606,7 @@ class Bag(DaskMethodsMixin):
         >>> b.starmap(myadd, z=max_second).compute()
         [13, 17, 21, 25, 29]
         """
-        name = "{0}-{1}".format(
-            funcname(func), tokenize(self, func, "starmap", **kwargs)
-        )
+        name = "{}-{}".format(funcname(func), tokenize(self, func, "starmap", **kwargs))
         dependencies = [self]
         if kwargs:
             kwargs, collections = unpack_scalar_dask_kwargs(kwargs)
@@ -638,11 +640,11 @@ class Bag(DaskMethodsMixin):
         >>> list(b.filter(iseven))
         [0, 2, 4]
         """
-        name = "filter-{0}-{1}".format(funcname(predicate), tokenize(self, predicate))
-        dsk = dict(
-            ((name, i), (reify, (filter, predicate, (self.name, i))))
+        name = f"filter-{funcname(predicate)}-{tokenize(self, predicate)}"
+        dsk = {
+            (name, i): (reify, (filter, predicate, (self.name, i)))
             for i in range(self.npartitions)
-        )
+        }
         graph = HighLevelGraph.from_collections(name, dsk, dependencies=[self])
         return type(self)(graph, name, self.npartitions)
 
@@ -692,11 +694,11 @@ class Bag(DaskMethodsMixin):
         >>> list(b.remove(iseven))
         [1, 3]
         """
-        name = "remove-{0}-{1}".format(funcname(predicate), tokenize(self, predicate))
-        dsk = dict(
-            ((name, i), (reify, (remove, predicate, (self.name, i))))
+        name = f"remove-{funcname(predicate)}-{tokenize(self, predicate)}"
+        dsk = {
+            (name, i): (reify, (remove, predicate, (self.name, i)))
             for i in range(self.npartitions)
-        )
+        }
         graph = HighLevelGraph.from_collections(name, dsk, dependencies=[self])
         return type(self)(graph, name, self.npartitions)
 
@@ -757,15 +759,15 @@ class Bag(DaskMethodsMixin):
         name = "pluck-" + tokenize(self, key, default)
         key = quote(key)
         if default == no_default:
-            dsk = dict(
-                ((name, i), (list, (pluck, key, (self.name, i))))
+            dsk = {
+                (name, i): (list, (pluck, key, (self.name, i)))
                 for i in range(self.npartitions)
-            )
+            }
         else:
-            dsk = dict(
-                ((name, i), (list, (pluck, key, (self.name, i), default)))
+            dsk = {
+                (name, i): (list, (pluck, key, (self.name, i), default))
                 for i in range(self.npartitions)
-            )
+            }
         graph = HighLevelGraph.from_collections(name, dsk, dependencies=[self])
         return type(self)(graph, name, self.npartitions)
 
@@ -798,7 +800,7 @@ class Bag(DaskMethodsMixin):
         compute=True,
         storage_options=None,
         last_endline=False,
-        **kwargs
+        **kwargs,
     ):
         return to_textfiles(
             self,
@@ -809,7 +811,7 @@ class Bag(DaskMethodsMixin):
             compute,
             storage_options=storage_options,
             last_endline=last_endline,
-            **kwargs
+            **kwargs,
         )
 
     @wraps(to_avro)
@@ -823,7 +825,7 @@ class Bag(DaskMethodsMixin):
         sync_interval=16000,
         metadata=None,
         compute=True,
-        **kwargs
+        **kwargs,
     ):
         return to_avro(
             self,
@@ -835,7 +837,7 @@ class Bag(DaskMethodsMixin):
             sync_interval,
             metadata,
             compute,
-            **kwargs
+            **kwargs,
         )
 
     def fold(
@@ -1009,7 +1011,7 @@ class Bag(DaskMethodsMixin):
             split_every = self.npartitions
 
         token = tokenize(self, perpartition, aggregate, split_every)
-        a = "%s-part-%s" % (name or funcname(perpartition), token)
+        a = f"{name or funcname(perpartition)}-part-{token}"
         is_last = self.npartitions == 1
         dsk = {
             (a, i): (empty_safe_apply, perpartition, (self.name, i), is_last)
@@ -1017,7 +1019,7 @@ class Bag(DaskMethodsMixin):
         }
         k = self.npartitions
         b = a
-        fmt = "%s-aggregate-%s" % (name or funcname(aggregate), token)
+        fmt = f"{name or funcname(aggregate)}-aggregate-{token}"
         depth = 0
 
         while k > split_every:
@@ -1061,15 +1063,39 @@ class Bag(DaskMethodsMixin):
         return self.reduction(min, min, split_every=split_every)
 
     def any(self, split_every=None):
-        """Are any of the elements truthy?"""
+        """Are any of the elements truthy?
+
+        Examples
+        --------
+        >>> import dask.bag as db
+        >>> bool_bag = db.from_sequence([True, True, False])
+        >>> bool_bag.any().compute()
+        True
+        """
         return self.reduction(any, any, split_every=split_every)
 
     def all(self, split_every=None):
-        """Are all elements truthy?"""
+        """Are all elements truthy?
+
+        Examples
+        --------
+        >>> import dask.bag as db
+        >>> bool_bag = db.from_sequence([True, True, False])
+        >>> bool_bag.all().compute()
+        False
+        """
         return self.reduction(all, all, split_every=split_every)
 
     def count(self, split_every=None):
-        """Count the number of elements."""
+        """Count the number of elements.
+
+        Examples
+        --------
+        >>> import dask.bag as db
+        >>> numbers = db.from_sequence([1, 2, 3])
+        >>> numbers.count().compute()
+        3
+        """
         return self.reduction(count, sum, split_every=split_every)
 
     def mean(self):
@@ -1172,14 +1198,14 @@ class Bag(DaskMethodsMixin):
         assert isinstance(other, Bag)
         name = "product-" + tokenize(self, other)
         n, m = self.npartitions, other.npartitions
-        dsk = dict(
-            (
-                (name, i * m + j),
-                (list, (itertools.product, (self.name, i), (other.name, j))),
+        dsk = {
+            (name, i * m + j): (
+                list,
+                (itertools.product, (self.name, i), (other.name, j)),
             )
             for i in range(n)
             for j in range(m)
-        )
+        }
         graph = HighLevelGraph.from_collections(name, dsk, dependencies=[self, other])
         return type(self)(graph, name, n * m)
 
@@ -1434,10 +1460,10 @@ class Bag(DaskMethodsMixin):
         [1, 2, 3]
         """
         name = "flatten-" + tokenize(self)
-        dsk = dict(
-            ((name, i), (list, (toolz.concat, (self.name, i))))
+        dsk = {
+            (name, i): (list, (toolz.concat, (self.name, i)))
             for i in range(self.npartitions)
-        )
+        }
         graph = HighLevelGraph.from_collections(name, dsk, dependencies=[self])
         return type(self)(graph, name, self.npartitions)
 
@@ -1449,7 +1475,7 @@ class Bag(DaskMethodsMixin):
         grouper,
         method=None,
         npartitions=None,
-        blocksize=2 ** 20,
+        blocksize=2**20,
         max_branch=None,
         shuffle=None,
     ):
@@ -1492,7 +1518,7 @@ class Bag(DaskMethodsMixin):
         if shuffle is None:
             shuffle = config.get("shuffle", None)
         if shuffle is None:
-            if "distributed" in config.get("scheduler", ""):
+            if config.get("scheduler", None) in ("dask.distributed", "distributed"):
                 shuffle = "tasks"
             else:
                 shuffle = "disk"
@@ -1506,7 +1532,7 @@ class Bag(DaskMethodsMixin):
             msg = "Shuffle must be 'disk' or 'tasks'"
             raise NotImplementedError(msg)
 
-    def to_dataframe(self, meta=None, columns=None):
+    def to_dataframe(self, meta=None, columns=None, optimize_graph=True):
         """Create Dask Dataframe from a Dask Bag.
 
         Bag should contain tuples, dict records, or scalars.
@@ -1534,6 +1560,10 @@ class Bag(DaskMethodsMixin):
             result (any names not found in the data will become all-NA
             columns).  Note that if ``meta`` is provided, column names will be
             taken from there and this parameter is invalid.
+        optimize_graph : bool, optional
+            If True [default], the graph is optimized before converting into
+            :class:`dask.dataframe.DataFrame`.
+
 
         Examples
         --------
@@ -1571,14 +1601,15 @@ class Bag(DaskMethodsMixin):
         # the empty frame
         cols = list(meta.columns)
         dtypes = meta.dtypes.to_dict()
-        name = "to_dataframe-" + tokenize(self, cols, dtypes)
-        dsk = self.__dask_optimize__(self.dask, self.__dask_keys__())
 
-        for i in range(self.npartitions):
-            dsk[(name, i)] = (to_dataframe, (self.name, i), cols, dtypes)
+        dfs = self.map_partitions(to_dataframe, cols, dtypes)
+        if optimize_graph:
+            dsk = self.__dask_optimize__(dfs.dask, dfs.__dask_keys__())
+        else:
+            dsk = dfs.dask
 
         divisions = [None] * (self.npartitions + 1)
-        return dd.DataFrame(dsk, name, meta, divisions)
+        return dd.DataFrame(dsk, dfs.name, meta, divisions)
 
     def to_delayed(self, optimize_graph=True):
         """Convert into a list of ``dask.delayed`` objects, one per partition.
@@ -1597,9 +1628,12 @@ class Bag(DaskMethodsMixin):
 
         keys = self.__dask_keys__()
         dsk = self.__dask_graph__()
+        layer = self.name
         if optimize_graph:
             dsk = self.__dask_optimize__(dsk, keys)
-        return [Delayed(k, dsk) for k in keys]
+            layer = "delayed-" + layer
+            dsk = HighLevelGraph.from_collections(layer, dsk, dependencies=())
+        return [Delayed(k, dsk, layer=layer) for k in keys]
 
     def repartition(self, npartitions=None, partition_size=None):
         """Repartition Bag across new divisions.
@@ -1657,9 +1691,9 @@ class Bag(DaskMethodsMixin):
         """
         token = tokenize(self, binop, initial)
         binop_name = funcname(binop)
-        a = "%s-part-%s" % (binop_name, token)
-        b = "%s-first-%s" % (binop_name, token)
-        c = "%s-second-%s" % (binop_name, token)
+        a = f"{binop_name}-part-{token}"
+        b = f"{binop_name}-first-{token}"
+        c = f"{binop_name}-second-{token}"
         dsk = {
             (a, 0): (accumulate_part, binop, (self.name, 0), initial, True),
             (b, 0): (first, (a, 0)),
@@ -1683,7 +1717,7 @@ def accumulate_part(binop, seq, initial, is_first=False):
     return res[1:], res[-1]
 
 
-def partition(grouper, sequence, npartitions, p, nelements=2 ** 20):
+def partition(grouper, sequence, npartitions, p, nelements=2**20):
     """Partition a bag along a grouper, store partitions on disk."""
     for block in partition_all(nelements, sequence):
         d = groupby(grouper, block)
@@ -1740,7 +1774,7 @@ def from_sequence(seq, partition_size=None, npartitions=None):
     parts = list(partition_all(partition_size, seq))
     name = "from_sequence-" + tokenize(seq, partition_size)
     if len(parts) > 0:
-        d = dict(((name, i), list(part)) for i, part in enumerate(parts))
+        d = {(name, i): list(part) for i, part in enumerate(parts)}
     else:
         d = {(name, 0): []}
 
@@ -1760,7 +1794,7 @@ def from_url(urls):
     (b'Dask\\n',
      b'====\\n',
      b'\\n',
-     b'|Build Status| |Coverage| |Doc Status| |Gitter| |Version Status| |NumFOCUS|\\n',
+     b'|Build Status| |Coverage| |Doc Status| |Discourse| |Version Status| |NumFOCUS|\\n',
      b'\\n',
      b'Dask is a flexible parallel computing library for analytics.  See\\n',
      b'documentation_ for more information.\\n',
@@ -1897,7 +1931,7 @@ def bag_range(n, npartitions):
     size = n // npartitions
     name = "range-%d-npartitions-%d" % (n, npartitions)
     ijs = list(enumerate(take(npartitions, range(0, n, size))))
-    dsk = dict(((name, i), (reify, (range, j, min(j + size, n)))) for i, j in ijs)
+    dsk = {(name, i): (reify, (range, j, min(j + size, n))) for i, j in ijs}
 
     if n % npartitions != 0:
         i, j = ijs[-1]
@@ -1949,10 +1983,10 @@ def bag_zip(*bags):
     # TODO: do more checks
 
     name = "zip-" + tokenize(*bags)
-    dsk = dict(
-        ((name, i), (reify, (zip,) + tuple((bag.name, i) for bag in bags)))
+    dsk = {
+        (name, i): (reify, (zip,) + tuple((bag.name, i) for bag in bags))
         for i in range(npartitions)
-    )
+    }
     graph = HighLevelGraph.from_collections(name, dsk, dependencies=bags)
     return Bag(graph, name, npartitions)
 
@@ -2099,8 +2133,7 @@ def bag_map(func, *args, **kwargs):
     >>> db.map(myadd, b, b.max()).compute()
     [4, 5, 6, 7, 8]
     """
-    name = "%s-%s" % (funcname(func), tokenize(func, "map", *args, **kwargs))
-    dsk = {}
+    name = "{}-{}".format(funcname(func), tokenize(func, "map", *args, **kwargs))
     dependencies = []
 
     bags = []
@@ -2198,12 +2231,11 @@ def map_partitions(func, *args, **kwargs):
     single graph, and then computes everything at once, and in some cases
     may be more efficient.
     """
-    name = "%s-%s" % (funcname(func), tokenize(func, "map-partitions", *args, **kwargs))
-    dsk = {}
-    dependencies = []
-
+    name = kwargs.pop("token", None) or funcname(func)
+    name = "{}-{}".format(name, tokenize(func, "map-partitions", *args, **kwargs))
     bags = []
     args2 = []
+    dependencies = []
     for a in args:
         if isinstance(a, Bag):
             bags.append(a)
@@ -2245,7 +2277,9 @@ def map_partitions(func, *args, **kwargs):
             (zip, list(bag_kwargs), [(b.name, n) for b in bag_kwargs.values()]),
         )
 
-    if kwargs:
+    if bag_kwargs:
+        # Avoid using `blockwise` when a key-word
+        # argument is being used to refer to a collection.
         dsk = {
             (name, n): (
                 apply,
@@ -2256,7 +2290,32 @@ def map_partitions(func, *args, **kwargs):
             for n in range(npartitions)
         }
     else:
-        dsk = {(name, n): (func,) + tuple(build_args(n)) for n in range(npartitions)}
+        pairs = []
+        numblocks = {}
+        for arg in args2:
+            # TODO: Allow interaction with Frame/Array
+            # collections with proper partitioning
+            if isinstance(arg, Bag):
+                pairs.extend([arg.name, "i"])
+                numblocks[arg.name] = (arg.npartitions,)
+            else:
+                pairs.extend([arg, None])
+        if other_kwargs and isinstance(other_kwargs, tuple):
+            # `other_kwargs` is a nested subgraph,
+            # designed to generate the kwargs lazily.
+            # We need to convert this to a dictionary
+            # before passing to `blockwise`
+            other_kwargs = other_kwargs[0](other_kwargs[1][0](*other_kwargs[1][1:]))
+        dsk = blockwise(
+            func,
+            name,
+            "i",
+            *pairs,
+            numblocks=numblocks,
+            concatenate=True,
+            dependencies=dependencies,
+            **other_kwargs,
+        )
 
     # If all bags are the same type, use that type, otherwise fallback to Bag
     return_type = set(map(type, bags))
@@ -2276,7 +2335,7 @@ def _reduce(binop, sequence, initial=no_default):
 
 def make_group(k, stage):
     def h(x):
-        return x[0] // k ** stage % k
+        return x[0] // k**stage % k
 
     return h
 
@@ -2295,7 +2354,7 @@ def groupby_tasks(b, grouper, hash=hash, max_branch=32):
     splits = []
     joins = []
 
-    inputs = [tuple(digit(i, j, k) for j in range(stages)) for i in range(k ** stages)]
+    inputs = [tuple(digit(i, j, k) for j in range(stages)) for i in range(k**stages)]
 
     b2 = b.map(partial(chunk.groupby_tasks_group_hash, hash=hash, grouper=grouper))
 
@@ -2335,39 +2394,33 @@ def groupby_tasks(b, grouper, hash=hash, max_branch=32):
         splits.append(split)
 
     for stage in range(1, stages + 1):
-        join = dict(
-            (
-                (shuffle_join_name, stage, inp),
+        join = {
+            (shuffle_join_name, stage, inp): (
+                list,
                 (
-                    list,
-                    (
-                        toolz.concat,
-                        [
-                            (
-                                shuffle_split_name,
-                                stage,
-                                inp[stage - 1],
-                                insert(inp, stage - 1, j),
-                            )
-                            for j in range(k)
-                        ],
-                    ),
+                    toolz.concat,
+                    [
+                        (
+                            shuffle_split_name,
+                            stage,
+                            inp[stage - 1],
+                            insert(inp, stage - 1, j),
+                        )
+                        for j in range(k)
+                    ],
                 ),
             )
             for inp in inputs
-        )
+        }
 
         joins.append(join)
 
     name = "shuffle-" + token
 
-    end = dict(
-        (
-            (name, i),
-            (list, (dict.items, (groupby, grouper, (pluck, 1, j)))),
-        )
+    end = {
+        (name, i): (list, (dict.items, (groupby, grouper, (pluck, 1, j))))
         for i, j in enumerate(join)
-    )
+    }
 
     groups.extend(splits)
     groups.extend(joins)
@@ -2377,7 +2430,7 @@ def groupby_tasks(b, grouper, hash=hash, max_branch=32):
     return type(b)(graph, name, len(inputs))
 
 
-def groupby_disk(b, grouper, npartitions=None, blocksize=2 ** 20):
+def groupby_disk(b, grouper, npartitions=None, blocksize=2**20):
     if npartitions is None:
         npartitions = b.npartitions
     token = tokenize(b, grouper, npartitions, blocksize)
@@ -2396,11 +2449,11 @@ def groupby_disk(b, grouper, npartitions=None, blocksize=2 ** 20):
         dsk1 = {p: (partd.Python, file)}
 
     # Partition data on disk
-    name = "groupby-part-{0}-{1}".format(funcname(grouper), token)
-    dsk2 = dict(
-        ((name, i), (partition, grouper, (b.name, i), npartitions, p, blocksize))
+    name = f"groupby-part-{funcname(grouper)}-{token}"
+    dsk2 = {
+        (name, i): (partition, grouper, (b.name, i), npartitions, p, blocksize)
         for i in range(b.npartitions)
-    )
+    }
 
     # Barrier
     barrier_token = "groupby-barrier-" + token
@@ -2409,9 +2462,9 @@ def groupby_disk(b, grouper, npartitions=None, blocksize=2 ** 20):
 
     # Collect groups
     name = "groupby-collect-" + token
-    dsk4 = dict(
-        ((name, i), (collect, grouper, i, p, barrier_token)) for i in range(npartitions)
-    )
+    dsk4 = {
+        (name, i): (collect, grouper, i, p, barrier_token) for i in range(npartitions)
+    }
 
     dsk = merge(dsk1, dsk2, dsk3, dsk4)
     graph = HighLevelGraph.from_collections(name, dsk, dependencies=[b])
@@ -2441,9 +2494,8 @@ def safe_take(n, b, warn=True):
     r = list(take(n, b))
     if len(r) != n and warn:
         warnings.warn(
-            "Insufficient elements for `take`. {0} elements "
-            "requested, only {1} elements available. Try passing "
-            "larger `npartitions` to `take`.".format(n, len(r))
+            f"Insufficient elements for `take`. {n} elements requested, only {len(r)} "
+            "elements available. Try passing larger `npartitions` to `take`."
         )
     return r
 
@@ -2564,8 +2616,8 @@ def repartition_size(bag, size):
 
     # 1. split each partition that is larger than partition size
     nsplits = [1 + mem_usage // size for mem_usage in mem_usages]
-    if any((nsplit > 1 for nsplit in nsplits)):
-        split_name = "repartition-split-{}".format(tokenize(bag, size))
+    if any(nsplit > 1 for nsplit in nsplits):
+        split_name = f"repartition-split-{tokenize(bag, size)}"
         bag = _split_partitions(bag, nsplits, split_name)
         # update mem_usages to account for the split partitions
         split_mem_usages = []
@@ -2574,10 +2626,10 @@ def repartition_size(bag, size):
         mem_usages = split_mem_usages
 
     # 2. now that all partitions are less than size, concat them up to size
-    assert all((mem_usage <= size for mem_usage in mem_usages))
+    assert all(mem_usage <= size for mem_usage in mem_usages)
     new_npartitions = list(map(len, iter_chunks(mem_usages, size)))
     new_partitions_boundaries = accumulate(operator.add, new_npartitions)
-    new_name = "repartition-{}".format(tokenize(bag, size))
+    new_name = f"repartition-{tokenize(bag, size)}"
     return _repartition_from_boundaries(bag, new_partitions_boundaries, new_name)
 
 
@@ -2598,9 +2650,9 @@ def _split_partitions(bag, nsplits, new_name):
     repartition_size
     """
     if len(nsplits) != bag.npartitions:
-        raise ValueError("nsplits should have len={}".format(bag.npartitions))
+        raise ValueError(f"nsplits should have len={bag.npartitions}")
     dsk = {}
-    split_name = "split-{}".format(tokenize(bag, nsplits))
+    split_name = f"split-{tokenize(bag, nsplits)}"
     j = 0
     for i, k in enumerate(nsplits):
         if k == 1:
