@@ -6,7 +6,7 @@ import pytest
 
 import dask.dataframe as dd
 from dask.base import compute_as_if_collection
-from dask.dataframe._compat import tm
+from dask.dataframe._compat import PANDAS_GT_140, tm
 from dask.dataframe.core import _Frame
 from dask.dataframe.methods import concat
 from dask.dataframe.multi import (
@@ -23,6 +23,7 @@ from dask.dataframe.utils import (
     has_known_categories,
     make_meta,
 )
+from dask.utils_test import hlg_layer_topological
 
 
 def test_align_partitions():
@@ -223,8 +224,7 @@ def list_eq(aa, bb):
 
 
 @pytest.mark.parametrize("how", ["inner", "left", "right", "outer"])
-@pytest.mark.parametrize("shuffle", ["disk", "tasks"])
-def test_hash_join(how, shuffle):
+def test_hash_join(how, shuffle_method):
     A = pd.DataFrame({"x": [1, 2, 3, 4, 5, 6], "y": [1, 1, 2, 2, 3, 4]})
     a = dd.repartition(A, [0, 4, 5])
 
@@ -238,7 +238,7 @@ def test_hash_join(how, shuffle):
     list_eq(result, expected)
 
     # Different columns and npartitions
-    c = hash_join(a, "x", b, "z", "outer", npartitions=3, shuffle=shuffle)
+    c = hash_join(a, "x", b, "z", "outer", npartitions=3, shuffle=shuffle_method)
     assert c.npartitions == 3
 
     result = c.compute(scheduler="single-threaded")
@@ -247,12 +247,12 @@ def test_hash_join(how, shuffle):
     list_eq(result, expected)
 
     assert (
-        hash_join(a, "y", b, "y", "inner", shuffle=shuffle)._name
-        == hash_join(a, "y", b, "y", "inner", shuffle=shuffle)._name
+        hash_join(a, "y", b, "y", "inner", shuffle=shuffle_method)._name
+        == hash_join(a, "y", b, "y", "inner", shuffle=shuffle_method)._name
     )
     assert (
-        hash_join(a, "y", b, "y", "inner", shuffle=shuffle)._name
-        != hash_join(a, "y", b, "y", "outer", shuffle=shuffle)._name
+        hash_join(a, "y", b, "y", "inner", shuffle=shuffle_method)._name
+        != hash_join(a, "y", b, "y", "outer", shuffle=shuffle_method)._name
     )
 
 
@@ -335,7 +335,7 @@ def test_merge_asof_left_on_right_index(
     b = dd.from_pandas(B, npartitions=2)
 
     if unknown_divisions:
-        a.divisions = [None] * len(a.divisions)
+        a.divisions = (None,) * len(a.divisions)
 
     C = pd.merge_asof(
         A,
@@ -368,7 +368,7 @@ def test_merge_asof_left_on_right_index(
             b = dd.from_pandas(B, npartitions=nparts)
 
             if unknown_divisions:
-                a.divisions = [None] * len(a.divisions)
+                a.divisions = (None,) * len(a.divisions)
 
             C = pd.merge_asof(
                 A,
@@ -753,24 +753,22 @@ def test_merge_columns_dtypes(how, on_index):
         b = b.set_index("A")
         on = None
 
-    with pytest.warns(None) as record:
+    with warnings.catch_warnings(record=True) as record:
+        warnings.simplefilter("always")
         result = dd.merge(
             a, b, on=on, how=how, left_index=left_index, right_index=right_index
         )
-
-    warned = any("merge column data type mismatches" in str(r) for r in record)
+        warned = any("merge column data type mismatches" in str(r) for r in record)
 
     # result type depends on merge operation -> convert to pandas
     result = result if isinstance(result, pd.DataFrame) else result.compute()
 
     has_nans = result.isna().values.any()
-
     assert (has_nans and warned) or not has_nans
 
 
 @pytest.mark.parametrize("how", ["inner", "outer", "left", "right"])
-@pytest.mark.parametrize("shuffle", ["disk", "tasks"])
-def test_merge(how, shuffle):
+def test_merge(how, shuffle_method):
     A = pd.DataFrame({"x": [1, 2, 3, 4, 5, 6], "y": [1, 1, 2, 2, 3, 4]})
     a = dd.repartition(A, [0, 4, 5])
 
@@ -778,7 +776,9 @@ def test_merge(how, shuffle):
     b = dd.repartition(B, [0, 2, 5])
 
     assert_eq(
-        dd.merge(a, b, left_index=True, right_index=True, how=how, shuffle=shuffle),
+        dd.merge(
+            a, b, left_index=True, right_index=True, how=how, shuffle=shuffle_method
+        ),
         pd.merge(A, B, left_index=True, right_index=True, how=how),
     )
 
@@ -787,7 +787,7 @@ def test_merge(how, shuffle):
     assert all(d is None for d in result.divisions)
 
     list_eq(
-        dd.merge(a, b, left_on="x", right_on="z", how=how, shuffle=shuffle),
+        dd.merge(a, b, left_on="x", right_on="z", how=how, shuffle=shuffle_method),
         pd.merge(A, B, left_on="x", right_on="z", how=how),
     )
     list_eq(
@@ -798,18 +798,20 @@ def test_merge(how, shuffle):
             right_on="z",
             how=how,
             suffixes=("1", "2"),
-            shuffle=shuffle,
+            shuffle=shuffle_method,
         ),
         pd.merge(A, B, left_on="x", right_on="z", how=how, suffixes=("1", "2")),
     )
 
-    list_eq(dd.merge(a, b, how=how, shuffle=shuffle), pd.merge(A, B, how=how))
-    list_eq(dd.merge(a, B, how=how, shuffle=shuffle), pd.merge(A, B, how=how))
-    list_eq(dd.merge(A, b, how=how, shuffle=shuffle), pd.merge(A, B, how=how))
-    list_eq(dd.merge(A, B, how=how, shuffle=shuffle), pd.merge(A, B, how=how))
+    list_eq(dd.merge(a, b, how=how, shuffle=shuffle_method), pd.merge(A, B, how=how))
+    list_eq(dd.merge(a, B, how=how, shuffle=shuffle_method), pd.merge(A, B, how=how))
+    list_eq(dd.merge(A, b, how=how, shuffle=shuffle_method), pd.merge(A, B, how=how))
+    list_eq(dd.merge(A, B, how=how, shuffle=shuffle_method), pd.merge(A, B, how=how))
 
     list_eq(
-        dd.merge(a, b, left_index=True, right_index=True, how=how, shuffle=shuffle),
+        dd.merge(
+            a, b, left_index=True, right_index=True, how=how, shuffle=shuffle_method
+        ),
         pd.merge(A, B, left_index=True, right_index=True, how=how),
     )
     list_eq(
@@ -820,13 +822,13 @@ def test_merge(how, shuffle):
             right_index=True,
             how=how,
             suffixes=("1", "2"),
-            shuffle=shuffle,
+            shuffle=shuffle_method,
         ),
         pd.merge(A, B, left_index=True, right_index=True, how=how, suffixes=("1", "2")),
     )
 
     list_eq(
-        dd.merge(a, b, left_on="x", right_index=True, how=how, shuffle=shuffle),
+        dd.merge(a, b, left_on="x", right_index=True, how=how, shuffle=shuffle_method),
         pd.merge(A, B, left_on="x", right_index=True, how=how),
     )
     list_eq(
@@ -837,7 +839,7 @@ def test_merge(how, shuffle):
             right_index=True,
             how=how,
             suffixes=("1", "2"),
-            shuffle=shuffle,
+            shuffle=shuffle_method,
         ),
         pd.merge(A, B, left_on="x", right_index=True, how=how, suffixes=("1", "2")),
     )
@@ -847,6 +849,26 @@ def test_merge(how, shuffle):
     #         pd.merge(A, B, left_index=True, right_on='y'))
 
 
+def test_merge_how_raises():
+    left = pd.DataFrame(
+        {
+            "A": ["A0", "A1", "A2", "A3"],
+            "B": ["B0", "B1", "B2", "B3"],
+        }
+    )
+    right = pd.DataFrame(
+        {
+            "A": ["C0", "C1", "C2", "C3"],
+            "B": ["D0", "D1", "D2", "D3"],
+        }
+    )
+
+    with pytest.raises(
+        ValueError, match="dask.dataframe.merge does not support how='cross'"
+    ):
+        dd.merge(left, right, how="cross")
+
+
 @pytest.mark.parametrize("parts", [(3, 3), (3, 1), (1, 3)])
 @pytest.mark.parametrize("how", ["leftsemi", "leftanti"])
 @pytest.mark.parametrize(
@@ -912,9 +934,8 @@ def test_merge_tasks_passes_through():
 
 
 @pytest.mark.slow
-@pytest.mark.parametrize("shuffle", ["disk", "tasks"])
 @pytest.mark.parametrize("how", ["inner", "outer", "left", "right"])
-def test_merge_by_index_patterns(how, shuffle):
+def test_merge_by_index_patterns(how, shuffle_method):
 
     pdf1l = pd.DataFrame({"a": [1, 2, 3, 4, 5, 6, 7], "b": [7, 6, 5, 4, 3, 2, 1]})
     pdf1r = pd.DataFrame({"c": [1, 2, 3, 4, 5, 6, 7], "d": [7, 6, 5, 4, 3, 2, 1]})
@@ -982,7 +1003,7 @@ def test_merge_by_index_patterns(how, shuffle):
                     how=how,
                     left_index=True,
                     right_index=True,
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                 ),
                 pd_merge(pdl, pdr, how=how, left_index=True, right_index=True),
             )
@@ -993,7 +1014,7 @@ def test_merge_by_index_patterns(how, shuffle):
                     how=how,
                     left_index=True,
                     right_index=True,
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                 ),
                 pd_merge(pdr, pdl, how=how, left_index=True, right_index=True),
             )
@@ -1005,7 +1026,7 @@ def test_merge_by_index_patterns(how, shuffle):
                     how=how,
                     left_index=True,
                     right_index=True,
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                     indicator=True,
                 ),
                 pd_merge(
@@ -1019,7 +1040,7 @@ def test_merge_by_index_patterns(how, shuffle):
                     how=how,
                     left_index=True,
                     right_index=True,
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                     indicator=True,
                 ),
                 pd_merge(
@@ -1029,24 +1050,36 @@ def test_merge_by_index_patterns(how, shuffle):
 
             assert_eq(
                 ddr.merge(
-                    ddl, how=how, left_index=True, right_index=True, shuffle=shuffle
+                    ddl,
+                    how=how,
+                    left_index=True,
+                    right_index=True,
+                    shuffle=shuffle_method,
                 ),
                 pdr.merge(pdl, how=how, left_index=True, right_index=True),
             )
             assert_eq(
                 ddl.merge(
-                    ddr, how=how, left_index=True, right_index=True, shuffle=shuffle
+                    ddr,
+                    how=how,
+                    left_index=True,
+                    right_index=True,
+                    shuffle=shuffle_method,
                 ),
                 pdl.merge(pdr, how=how, left_index=True, right_index=True),
             )
 
             # hash join
             list_eq(
-                dd.merge(ddl, ddr, how=how, left_on="a", right_on="c", shuffle=shuffle),
+                dd.merge(
+                    ddl, ddr, how=how, left_on="a", right_on="c", shuffle=shuffle_method
+                ),
                 pd.merge(pdl, pdr, how=how, left_on="a", right_on="c"),
             )
             list_eq(
-                dd.merge(ddl, ddr, how=how, left_on="b", right_on="d", shuffle=shuffle),
+                dd.merge(
+                    ddl, ddr, how=how, left_on="b", right_on="d", shuffle=shuffle_method
+                ),
                 pd.merge(pdl, pdr, how=how, left_on="b", right_on="d"),
             )
 
@@ -1057,7 +1090,7 @@ def test_merge_by_index_patterns(how, shuffle):
                     how=how,
                     left_on="c",
                     right_on="a",
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                     indicator=True,
                 ),
                 pd.merge(pdr, pdl, how=how, left_on="c", right_on="a", indicator=True),
@@ -1069,43 +1102,54 @@ def test_merge_by_index_patterns(how, shuffle):
                     how=how,
                     left_on="d",
                     right_on="b",
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                     indicator=True,
                 ),
                 pd.merge(pdr, pdl, how=how, left_on="d", right_on="b", indicator=True),
             )
 
             list_eq(
-                dd.merge(ddr, ddl, how=how, left_on="c", right_on="a", shuffle=shuffle),
+                dd.merge(
+                    ddr, ddl, how=how, left_on="c", right_on="a", shuffle=shuffle_method
+                ),
                 pd.merge(pdr, pdl, how=how, left_on="c", right_on="a"),
             )
             list_eq(
-                dd.merge(ddr, ddl, how=how, left_on="d", right_on="b", shuffle=shuffle),
+                dd.merge(
+                    ddr, ddl, how=how, left_on="d", right_on="b", shuffle=shuffle_method
+                ),
                 pd.merge(pdr, pdl, how=how, left_on="d", right_on="b"),
             )
 
             list_eq(
-                ddl.merge(ddr, how=how, left_on="a", right_on="c", shuffle=shuffle),
+                ddl.merge(
+                    ddr, how=how, left_on="a", right_on="c", shuffle=shuffle_method
+                ),
                 pdl.merge(pdr, how=how, left_on="a", right_on="c"),
             )
             list_eq(
-                ddl.merge(ddr, how=how, left_on="b", right_on="d", shuffle=shuffle),
+                ddl.merge(
+                    ddr, how=how, left_on="b", right_on="d", shuffle=shuffle_method
+                ),
                 pdl.merge(pdr, how=how, left_on="b", right_on="d"),
             )
 
             list_eq(
-                ddr.merge(ddl, how=how, left_on="c", right_on="a", shuffle=shuffle),
+                ddr.merge(
+                    ddl, how=how, left_on="c", right_on="a", shuffle=shuffle_method
+                ),
                 pdr.merge(pdl, how=how, left_on="c", right_on="a"),
             )
             list_eq(
-                ddr.merge(ddl, how=how, left_on="d", right_on="b", shuffle=shuffle),
+                ddr.merge(
+                    ddl, how=how, left_on="d", right_on="b", shuffle=shuffle_method
+                ),
                 pdr.merge(pdl, how=how, left_on="d", right_on="b"),
             )
 
 
 @pytest.mark.parametrize("how", ["inner", "outer", "left", "right"])
-@pytest.mark.parametrize("shuffle", ["disk", "tasks"])
-def test_join_by_index_patterns(how, shuffle):
+def test_join_by_index_patterns(how, shuffle_method):
 
     # Similar test cases as test_merge_by_index_patterns,
     # but columns / index for join have same dtype
@@ -1161,15 +1205,23 @@ def test_join_by_index_patterns(how, shuffle):
             ddl = dd.from_pandas(pdl, lpart)
             ddr = dd.from_pandas(pdr, rpart)
 
-            assert_eq(ddl.join(ddr, how=how, shuffle=shuffle), pdl.join(pdr, how=how))
-            assert_eq(ddr.join(ddl, how=how, shuffle=shuffle), pdr.join(pdl, how=how))
+            assert_eq(
+                ddl.join(ddr, how=how, shuffle=shuffle_method), pdl.join(pdr, how=how)
+            )
+            assert_eq(
+                ddr.join(ddl, how=how, shuffle=shuffle_method), pdr.join(pdl, how=how)
+            )
 
             assert_eq(
-                ddl.join(ddr, how=how, lsuffix="l", rsuffix="r", shuffle=shuffle),
+                ddl.join(
+                    ddr, how=how, lsuffix="l", rsuffix="r", shuffle=shuffle_method
+                ),
                 pdl.join(pdr, how=how, lsuffix="l", rsuffix="r"),
             )
             assert_eq(
-                ddr.join(ddl, how=how, lsuffix="l", rsuffix="r", shuffle=shuffle),
+                ddr.join(
+                    ddl, how=how, lsuffix="l", rsuffix="r", shuffle=shuffle_method
+                ),
                 pdr.join(pdl, how=how, lsuffix="l", rsuffix="r"),
             )
 
@@ -1211,8 +1263,7 @@ def test_join_gives_proper_divisions():
 
 
 @pytest.mark.parametrize("how", ["inner", "outer", "left", "right"])
-@pytest.mark.parametrize("shuffle", ["disk", "tasks"])
-def test_merge_by_multiple_columns(how, shuffle):
+def test_merge_by_multiple_columns(how, shuffle_method):
     # warnings here from pandas
     pdf1l = pd.DataFrame(
         {
@@ -1272,8 +1323,12 @@ def test_merge_by_multiple_columns(how, shuffle):
             ddl = dd.from_pandas(pdl, lpart)
             ddr = dd.from_pandas(pdr, rpart)
 
-            assert_eq(ddl.join(ddr, how=how, shuffle=shuffle), pdl.join(pdr, how=how))
-            assert_eq(ddr.join(ddl, how=how, shuffle=shuffle), pdr.join(pdl, how=how))
+            assert_eq(
+                ddl.join(ddr, how=how, shuffle=shuffle_method), pdl.join(pdr, how=how)
+            )
+            assert_eq(
+                ddr.join(ddl, how=how, shuffle=shuffle_method), pdr.join(pdl, how=how)
+            )
 
             assert_eq(
                 dd.merge(
@@ -1282,7 +1337,7 @@ def test_merge_by_multiple_columns(how, shuffle):
                     how=how,
                     left_index=True,
                     right_index=True,
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                 ),
                 pd.merge(pdl, pdr, how=how, left_index=True, right_index=True),
             )
@@ -1293,27 +1348,35 @@ def test_merge_by_multiple_columns(how, shuffle):
                     how=how,
                     left_index=True,
                     right_index=True,
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                 ),
                 pd.merge(pdr, pdl, how=how, left_index=True, right_index=True),
             )
 
             # hash join
             list_eq(
-                dd.merge(ddl, ddr, how=how, left_on="a", right_on="d", shuffle=shuffle),
+                dd.merge(
+                    ddl, ddr, how=how, left_on="a", right_on="d", shuffle=shuffle_method
+                ),
                 pd.merge(pdl, pdr, how=how, left_on="a", right_on="d"),
             )
             list_eq(
-                dd.merge(ddl, ddr, how=how, left_on="b", right_on="e", shuffle=shuffle),
+                dd.merge(
+                    ddl, ddr, how=how, left_on="b", right_on="e", shuffle=shuffle_method
+                ),
                 pd.merge(pdl, pdr, how=how, left_on="b", right_on="e"),
             )
 
             list_eq(
-                dd.merge(ddr, ddl, how=how, left_on="d", right_on="a", shuffle=shuffle),
+                dd.merge(
+                    ddr, ddl, how=how, left_on="d", right_on="a", shuffle=shuffle_method
+                ),
                 pd.merge(pdr, pdl, how=how, left_on="d", right_on="a"),
             )
             list_eq(
-                dd.merge(ddr, ddl, how=how, left_on="e", right_on="b", shuffle=shuffle),
+                dd.merge(
+                    ddr, ddl, how=how, left_on="e", right_on="b", shuffle=shuffle_method
+                ),
                 pd.merge(pdr, pdl, how=how, left_on="e", right_on="b"),
             )
 
@@ -1324,7 +1387,7 @@ def test_merge_by_multiple_columns(how, shuffle):
                     how=how,
                     left_on=["a", "b"],
                     right_on=["d", "e"],
-                    shuffle=shuffle,
+                    shuffle=shuffle_method,
                 ),
                 pd.merge(pdl, pdr, how=how, left_on=["a", "b"], right_on=["d", "e"]),
             )
@@ -1372,12 +1435,14 @@ def test_cheap_inner_merge_with_pandas_object():
     b = pd.DataFrame({"x": [1, 2, 3, 4], "z": list("abda")})
 
     dc = da.merge(b, on="x", how="inner")
+    assert not hlg_layer_topological(dc.dask, -1).is_materialized()
     assert all("shuffle" not in k[0] for k in dc.dask)
 
     list_eq(da.merge(b, on="x", how="inner"), a.merge(b, on="x", how="inner"))
 
 
-def test_cheap_single_partition_merge():
+@pytest.mark.parametrize("flip", [False, True])
+def test_cheap_single_partition_merge(flip):
     a = pd.DataFrame(
         {"x": [1, 2, 3, 4, 5, 6], "y": list("abdabd")}, index=[10, 20, 30, 40, 50, 60]
     )
@@ -1386,11 +1451,15 @@ def test_cheap_single_partition_merge():
     b = pd.DataFrame({"x": [1, 2, 3, 4], "z": list("abda")})
     bb = dd.from_pandas(b, npartitions=1, sort=False)
 
-    cc = aa.merge(bb, on="x", how="inner")
+    pd_inputs = (b, a) if flip else (a, b)
+    inputs = (bb, aa) if flip else (aa, bb)
+
+    cc = dd.merge(*inputs, on="x", how="inner")
+    assert not hlg_layer_topological(cc.dask, -1).is_materialized()
     assert all("shuffle" not in k[0] for k in cc.dask)
     assert len(cc.dask) == len(aa.dask) * 2 + len(bb.dask)
 
-    list_eq(aa.merge(bb, on="x", how="inner"), a.merge(b, on="x", how="inner"))
+    list_eq(cc, pd.merge(*pd_inputs, on="x", how="inner"))
 
 
 def test_cheap_single_partition_merge_divisions():
@@ -1403,6 +1472,8 @@ def test_cheap_single_partition_merge_divisions():
     bb = dd.from_pandas(b, npartitions=1, sort=False)
 
     actual = aa.merge(bb, on="x", how="inner")
+    assert not hlg_layer_topological(actual.dask, -1).is_materialized()
+
     assert not actual.known_divisions
     assert_divisions(actual)
 
@@ -1412,21 +1483,27 @@ def test_cheap_single_partition_merge_divisions():
 
 
 @pytest.mark.parametrize("how", ["left", "right"])
-def test_cheap_single_parition_merge_left_right(how):
+@pytest.mark.parametrize("flip", [False, True])
+def test_cheap_single_parition_merge_left_right(how, flip):
     a = pd.DataFrame({"x": range(8), "z": list("ababbdda")}, index=range(8))
     aa = dd.from_pandas(a, npartitions=1)
 
     b = pd.DataFrame({"x": [1, 2, 3, 4], "z": list("abda")}, index=range(4))
     bb = dd.from_pandas(b, npartitions=1)
 
-    actual = aa.merge(bb, left_index=True, right_on="x", how=how)
-    expected = a.merge(b, left_index=True, right_on="x", how=how)
+    pd_inputs = (b, a) if flip else (a, b)
+    inputs = (bb, aa) if flip else (aa, bb)
 
+    actual = dd.merge(*inputs, left_index=True, right_on="x", how=how)
+    expected = pd.merge(*pd_inputs, left_index=True, right_on="x", how=how)
+
+    assert not hlg_layer_topological(actual.dask, -1).is_materialized()
     assert_eq(actual, expected)
 
-    actual = aa.merge(bb, left_on="x", right_index=True, how=how)
-    expected = a.merge(b, left_on="x", right_index=True, how=how)
+    actual = dd.merge(*inputs, left_on="x", right_index=True, how=how)
+    expected = pd.merge(*pd_inputs, left_on="x", right_index=True, how=how)
 
+    assert not hlg_layer_topological(actual.dask, -1).is_materialized()
     assert_eq(actual, expected)
 
 
@@ -1447,6 +1524,7 @@ def test_cheap_single_partition_merge_on_index():
     # for empty joins.
     expected.index = expected.index.astype("int64")
 
+    assert not hlg_layer_topological(actual.dask, -1).is_materialized()
     assert not actual.known_divisions
     assert_eq(actual, expected)
 
@@ -1454,6 +1532,7 @@ def test_cheap_single_partition_merge_on_index():
     expected = b.merge(a, right_index=True, left_on="x", how="inner")
     expected.index = expected.index.astype("int64")
 
+    assert not hlg_layer_topological(actual.dask, -1).is_materialized()
     assert not actual.known_divisions
     assert_eq(actual, expected)
 
@@ -1470,15 +1549,14 @@ def test_merge_maintains_columns():
     assert tuple(merged.columns) == ("D", "C", "B", "A", "G", "H", "I")
 
 
-@pytest.mark.parametrize("shuffle", ["disk", "tasks"])
-def test_merge_index_without_divisions(shuffle):
+def test_merge_index_without_divisions(shuffle_method):
     a = pd.DataFrame({"x": [1, 2, 3, 4, 5]}, index=[1, 2, 3, 4, 5])
     b = pd.DataFrame({"y": [1, 2, 3, 4, 5]}, index=[5, 4, 3, 2, 1])
 
     aa = dd.from_pandas(a, npartitions=3, sort=False)
     bb = dd.from_pandas(b, npartitions=2)
 
-    result = aa.join(bb, how="inner", shuffle=shuffle)
+    result = aa.join(bb, how="inner", shuffle=shuffle_method)
     expected = a.join(b, how="inner")
     assert_eq(result, expected)
 
@@ -1540,9 +1618,9 @@ def test_concat_unknown_divisions():
     with pytest.raises(ValueError):
         dd.concat([aa, cc], axis=1)
 
-    with pytest.warns(None) as record:
+    with warnings.catch_warnings(record=True) as record:
         dd.concat([aa, bb], axis=1, ignore_unknown_divisions=True)
-        assert len(record) == 0
+    assert not record
 
 
 def test_concat_unknown_divisions_errors():
@@ -1778,12 +1856,10 @@ def test_concat5():
     for case in cases:
         pdcase = [c.compute() for c in case]
 
-        with pytest.warns(None):
-            # some cases will raise warning directly from pandas
-            assert_eq(
-                dd.concat(case, interleave_partitions=True),
-                pd.concat(pdcase, sort=False),
-            )
+        assert_eq(
+            dd.concat(case, interleave_partitions=True),
+            pd.concat(pdcase, sort=False),
+        )
 
         assert_eq(
             dd.concat(case, join="inner", interleave_partitions=True),
@@ -1950,6 +2026,20 @@ def test_concat_datetimeindex():
     assert_eq(result, expected)
 
 
+def check_append_with_warning(dask_obj, dask_append, pandas_obj, pandas_append):
+    if PANDAS_GT_140:
+        with pytest.warns(FutureWarning, match="append method is deprecated"):
+            expected = pandas_obj.append(pandas_append)
+            result = dask_obj.append(dask_append)
+            assert_eq(result, expected)
+    else:
+        expected = pandas_obj.append(pandas_append)
+        result = dask_obj.append(dask_append)
+        assert_eq(result, expected)
+
+    return result
+
+
 def test_append():
     df = pd.DataFrame({"a": [1, 2, 3, 4, 5, 6], "b": [1, 2, 3, 4, 5, 6]})
     df2 = pd.DataFrame(
@@ -1965,29 +2055,20 @@ def test_append():
 
     s = pd.Series([7, 8], name=6, index=["a", "b"])
 
-    def check_with_warning(dask_obj, dask_append, pandas_obj, pandas_append):
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", FutureWarning)
-            expected = pandas_obj.append(pandas_append)
-
-            result = dask_obj.append(dask_append)
-
-        assert_eq(result, expected)
-
-    check_with_warning(ddf, s, df, s)
-    check_with_warning(ddf, ddf2, df, df2)
-    check_with_warning(ddf.a, ddf2.a, df.a, df2.a)
+    check_append_with_warning(ddf, s, df, s)
+    check_append_with_warning(ddf, ddf2, df, df2)
+    check_append_with_warning(ddf.a, ddf2.a, df.a, df2.a)
 
     # different columns
-    check_with_warning(ddf, ddf3, df, df3)
-    check_with_warning(ddf.a, ddf3.b, df.a, df3.b)
+    check_append_with_warning(ddf, ddf3, df, df3)
+    check_append_with_warning(ddf.a, ddf3.b, df.a, df3.b)
 
     # dask + pandas
-    check_with_warning(ddf, df2, df, df2)
-    check_with_warning(ddf.a, df2.a, df.a, df2.a)
+    check_append_with_warning(ddf, df2, df, df2)
+    check_append_with_warning(ddf.a, df2.a, df.a, df2.a)
 
-    check_with_warning(ddf, df3, df, df3)
-    check_with_warning(ddf.a, df3.b, df.a, df3.b)
+    check_append_with_warning(ddf, df3, df, df3)
+    check_append_with_warning(ddf.a, df3.b, df.a, df3.b)
 
 
 def test_append2():
@@ -1998,6 +2079,7 @@ def test_append2():
     }
     meta = make_meta({"a": "i8", "b": "i8"}, parent_meta=pd.DataFrame())
     ddf1 = dd.DataFrame(dsk, "x", meta, [None, None])
+    df1 = ddf1.compute()
 
     dsk = {
         ("y", 0): pd.DataFrame({"a": [10, 20, 30], "b": [40, 50, 60]}),
@@ -2005,6 +2087,7 @@ def test_append2():
         ("y", 2): pd.DataFrame({"a": [70, 80, 90], "b": [0, 0, 0]}),
     }
     ddf2 = dd.DataFrame(dsk, "y", meta, [None, None])
+    df2 = ddf2.compute()
 
     dsk = {
         ("y", 0): pd.DataFrame({"b": [10, 20, 30], "c": [40, 50, 60]}),
@@ -2012,29 +2095,22 @@ def test_append2():
     }
     meta = make_meta({"b": "i8", "c": "i8"}, parent_meta=pd.DataFrame())
     ddf3 = dd.DataFrame(dsk, "y", meta, [None, None])
+    df3 = ddf3.compute()
 
-    assert_eq(ddf1.append(ddf2), ddf1.compute().append(ddf2.compute(), sort=False))
-    assert_eq(ddf2.append(ddf1), ddf2.compute().append(ddf1.compute(), sort=False))
+    check_append_with_warning(ddf1, ddf2, df1, df2)
+    check_append_with_warning(ddf2, ddf1, df2, df1)
 
     # different columns
-    assert_eq(ddf1.append(ddf3), ddf1.compute().append(ddf3.compute(), sort=False))
-    assert_eq(ddf3.append(ddf1), ddf3.compute().append(ddf1.compute(), sort=False))
+    check_append_with_warning(ddf1, ddf3, df1, df3)
+    check_append_with_warning(ddf3, ddf1, df3, df1)
 
     # Dask + pandas
-    assert_eq(
-        ddf1.append(ddf2.compute()), ddf1.compute().append(ddf2.compute(), sort=False)
-    )
-    assert_eq(
-        ddf2.append(ddf1.compute()), ddf2.compute().append(ddf1.compute(), sort=False)
-    )
+    check_append_with_warning(ddf1, df2, df1, df2)
+    check_append_with_warning(ddf2, df1, df2, df1)
 
     # different columns
-    assert_eq(
-        ddf1.append(ddf3.compute()), ddf1.compute().append(ddf3.compute(), sort=False)
-    )
-    assert_eq(
-        ddf3.append(ddf1.compute()), ddf3.compute().append(ddf1.compute(), sort=False)
-    )
+    check_append_with_warning(ddf1, df3, df1, df3)
+    check_append_with_warning(ddf3, df1, df3, df1)
 
 
 def test_append_categorical():
@@ -2072,18 +2148,16 @@ def test_append_categorical():
             )
         ddf1, ddf2 = dframes
 
-        res = ddf1.append(ddf2)
-        assert_eq(res, df1.append(df2))
+        res = check_append_with_warning(ddf1, ddf2, df1, df2)
+
         assert has_known_categories(res.index) == known
         assert has_known_categories(res.y) == known
 
-        res = ddf1.y.append(ddf2.y)
-        assert_eq(res, df1.y.append(df2.y))
+        res = check_append_with_warning(ddf1.y, ddf2.y, df1.y, df2.y)
         assert has_known_categories(res.index) == known
         assert has_known_categories(res) == known
 
-        res = ddf1.index.append(ddf2.index)
-        assert_eq(res, df1.index.append(df2.index))
+        res = check_append_with_warning(ddf1.index, ddf2.index, df1.index, df2.index)
         assert has_known_categories(res) == known
 
 
@@ -2091,9 +2165,8 @@ def test_append_lose_divisions():
     df = pd.DataFrame({"x": [1, 2, 3, 4]}, index=[1, 2, 3, 4])
     ddf = dd.from_pandas(df, npartitions=2)
 
-    ddf2 = ddf.append(ddf)
-    df2 = df.append(df)
-    assert_eq(ddf2, df2)
+    res = check_append_with_warning(ddf, ddf, df, df)
+    assert res.known_divisions is False
 
 
 def test_singleton_divisions():
@@ -2155,10 +2228,9 @@ def test_dtype_equality_warning():
     df1 = pd.DataFrame({"a": np.array([1, 2], dtype=np.dtype(np.int64))})
     df2 = pd.DataFrame({"a": np.array([1, 2], dtype=np.dtype(np.longlong))})
 
-    with pytest.warns(None) as r:
+    with warnings.catch_warnings(record=True) as record:
         dd.multi.warn_dtype_mismatch(df1, df2, "a", "a")
-
-    assert len(r) == 0
+    assert not record
 
 
 @pytest.mark.parametrize(
